#include "Toolbar.h"
#include "Mask.h"

LRESULT CALLBACK windowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    char filePath[2048];
    char buffer[2048]; 
    int ret;

    switch (message) {
        case WM_COMMAND:
            if( programState == STATE_FREE) {
                if( maskHandledEvent(LOWORD(wParam)) ) maskCommandEvent( LOWORD(wParam) );
                switch(LOWORD(wParam)) {
                    case ID_IMGPATH_BUTTON:
                        if( windowOpenFileDialog( filePath ) ) {
                            windowImagePath = string( filePath );
                            UISetEditText( filePath, hImgPathEdit );
                            windowLoadImageMask();
                            windowPrepareInpaint();
                        }
                        break;
                    case ID_MSKPATH_BUTTON:
                        if( windowOpenFileDialog( filePath ) ) {
                            windowMaskPath = string( filePath );
                            UISetEditText( filePath, hMskPathEdit );
                            if( windowImagePath != "" ) {
                                windowLoadImageMask();
                                windowPrepareInpaint();
                            }
                        }
                        break;
                    case ID_OUTPATH_BUTTON:
                        if( windowSaveFileDialog( filePath ) ) {
                            windowOutputPath = string( filePath );
                            UISetEditText( filePath, hOutPathEdit );
                        }
                        break;
                    case ID_RUN_BUTTON:
                        // Run instant-result algorithms
                        if( windowSelectedAlgorithm == 0 ||
                            windowSelectedAlgorithm == 3 ||
                            windowSelectedAlgorithm == 4 ) { 
                            printl("Running algorithm...");
                            windowChangeProgramState( STATE_RUN );
                            windowPrepareInpaint();
                            globalInpainter.inpaint();
                            windowChangeProgramState( STATE_FREE );
                            printl("Done...");
                            windowUpdateResult();
                        } else {
                            windowPrepareInpaint();
                            windowChangeProgramState( STATE_RUN );
                        }
                        break;
                    case ID_RESET_BUTTON:
                        windowPrepareInpaint();
                        windowUpdateResult();
                        break;
                    case ID_ALG_COMBO:
                        if( HIWORD(wParam) == CBN_SELCHANGE) {
                            ret = UIGetSelCombo( hAlgSelectCombo );
                            if( ret != CB_ERR) {
                                windowSelectedAlgorithm = ret;
                                windowParametersSwitchAlgorithm();
                                windowPrepareInpaint();
                            }
                        }
                        break;
                    case ID_SEP_COMBO:
                        if( HIWORD(wParam) == CBN_SELCHANGE) {
                            ret = UIGetSelCombo( hSepSelectCombo );
                            if( ret != CB_ERR) {
                                windowSelectedSeparation = ret;
                                windowPrepareInpaint();
                            }
                        }
                        break;
                    case ID_FDII_KERNEL_COMBO:
                        if( HIWORD(wParam) == CBN_SELCHANGE) {
                            ret = UIGetSelCombo( hFDIIKernelCombo );
                            if( ret != CB_ERR) {
                                globalInpainter.pFDIIKernel = ret;
                            }
                        }
                        break;
                    case ID_EXP_GPU_CHECK:
                        globalInpainter.pExmpGPU = UIGetCheckBox( hExmpGPUCheck );
                        break;
                    case ID_MASK_CHECK:
                        ret = UIGetCheckBox( hMaskEditCheck );
                        ShowWindow( hMaskEditWindow, ret ? SW_SHOW : SW_HIDE );
                        programMaskEdit = ret;
                        windowUpdateResult();
                        break;
                    case ID_SAVE_CHECK:
                        printf("Save!\n");
                        ret = UIGetCheckBox( hSaveImgCheck );
                        windowSaveInterim = ret;
                        break;
                    case ID_HELP_CHECK:
                        ret = UIGetCheckBox( hHelpShowCheck );
                        break;
                }
            } else {
                if( LOWORD(wParam) == ID_RUN_BUTTON ) {
                    windowChangeProgramState( STATE_FREE );
                    windowUpdateResult();
                } else if( LOWORD(wParam) == ID_RESET_BUTTON ) {
                    windowPrepareInpaint();
                    windowChangeProgramState( STATE_FREE );
                    windowUpdateResult();
                }
            }
            break;
        case WM_HSCROLL:
            if( programState == STATE_FREE) {
                    globalInpainter.pGaussianOctaves = UIGetTrackBar( hGaussianOctavesTrackbar );
                    UISetTextPrintf( hGaussianOctavesLabel, "Octaves: %d", 
                                     globalInpainter.pGaussianOctaves);
                    globalInpainter.pBertalmioInpaintSteps = UIGetTrackBar( hBertalmioInpaintStepsTrackbar );
                    globalInpainter.pBertalmioDiffuseSteps = UIGetTrackBar( hBertalmioDiffuseStepsTrackbar );
                    UISetTextPrintf( hBertalmioInpaintStepsLabel, "Inpaint Steps: %d", 
                                     globalInpainter.pBertalmioInpaintSteps);
                    UISetTextPrintf( hBertalmioDiffuseStepsLabel, "Diffusion Steps: %d", 
                                     globalInpainter.pBertalmioDiffuseSteps);
                    globalInpainter.pTeleaRadius = UIGetTrackBar( hTeleaRadiusTrackbar );
                    globalInpainter.pNavierRadius = UIGetTrackBar( hNavierRadiusTrackbar );
                    UISetTextPrintf( hTeleaRadiusLabel, "Radius: %d", globalInpainter.pTeleaRadius);
                    UISetTextPrintf( hNavierRadiusLabel, "Radius: %d", globalInpainter.pNavierRadius);
                    globalInpainter.pFDIIIterations = UIGetTrackBar( hFDIIIterTrackbar );
                    UISetTextPrintf( hFDIIIterLabel, "Iterations: %d", globalInpainter.pFDIIIterations);
                    globalInpainter.pExmpIterations = UIGetTrackBar( hExmpIterTrackbar );
                    UISetTextPrintf( hExmpIterLabel, "Iterations: %d", globalInpainter.pExmpIterations);
                    globalInpainter.pExmpPatchSize = UIGetTrackBar( hExmpPatchSizeTrackbar );
                    UISetTextPrintf( hExmpPatchSizeLabel, "Patch Size: %d", globalInpainter.pExmpPatchSize * 2 + 1);
                    globalInpainter.pExmpBoundary = UIGetTrackBar( hExmpBoundaryTrackbar );
                    UISetTextPrintf( hExmpBoundaryLabel, "Boundary Radius: %d", globalInpainter.pExmpBoundary * 3);
                    maskUpdateTrackbars();
            }
            break;
        case WM_DESTROY:
            programQuit = true;
            PostQuitMessage(0);
            exit(0);
            break;
        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }
}