#include "Parallel.h"

#ifdef _GPU_
    bool gpuEnabled = true;
#else
    bool gpuEnabled = false;
#endif

#ifdef _GPU_

extern "C" float *gpuComputeDistance(int2 *hstSourcePatches, float3 *hstImage, char *hstMask,
                                     int height, int width, int list, int patchSize, int px, int py );

float *gpuComputeDistanceList(IplImage *img, IplImage *msk, int patchSize,
                              const vector< pair<int, int> >& sourcePatches,
                              int px, int py) {
    if( !img || !msk ) return NULL;
    if( sourcePatches.size() <= 0 ) return NULL;
    if( img->width != msk->width || img->height != msk->height ) return NULL;
    
    int numPixels = img->height * img->width ;

    //printf("HOST: Assigning memory...\n");
    int2 *hstSourcePatches = new int2[sourcePatches.size()];
    float3 *hstImage = new float3[numPixels];
    char *hstMask = new char[numPixels];

    if(!hstSourcePatches) return NULL;
    if(!hstImage) return NULL;
    if(!hstMask) return NULL;
 
    //printf("HOST: copying source paches...\n");
    for(int a = 0; a < sourcePatches.size(); a++) {
        hstSourcePatches[a].x = sourcePatches[a].first;
        hstSourcePatches[a].y = sourcePatches[a].second;
    }
    
    //printf("HOST: copying images...\n");
    for(int a = 0; a < img->height; a++) {
        float3 *hstImageRow = &hstImage [ a * img->width ];
        char *hskMaskRow = &hstMask [ a * img->width ];
        for(int b = 0; b < img->width; b++) {
            CvScalar pixel = cvGet2D( img, a, b );
            hstImageRow[b].x = pixel.val[0];
            hstImageRow[b].y = pixel.val[1];
            hstImageRow[b].z = pixel.val[2];
            CvScalar maskpixel = cvGet2D( msk, a, b );
            hskMaskRow[b] = (maskpixel.val[0] > 128) ? 10 : 0;
        }
    }
 
    //printf("HOST: Calling gpuComputeDistance...\n");
    float *resultList = gpuComputeDistance( hstSourcePatches, hstImage, hstMask, img->height, img->width, 
                                            sourcePatches.size(), patchSize, px, py );
    if( !resultList ) return NULL;

    //printf("HOST: Getting results...\n");

    delete[] hstSourcePatches;
    delete[] hstImage;
    delete[] hstMask;

    return resultList;
}

#endif

