#include "Toolbar.h"
#include "Diffusion.h"

// --> Control Variables
bool programQuit = false;
int programState = STATE_FREE;
bool programMaskEdit = false;

char comboAlgorithmOptions [][64] = {
    "Gaussian (Blur)",
    "FDII (Blur)",
    "Bertalmio (PDE)",
    "Telea (PDE)",
    "Navier-Stokes (PDE)",
    "Exemplar (TS)",
    "Local Optimisation (TS)"
};

char comboSeparationOptions [][64] = {
    "Greyscale",
    "Desaturate",
    "RGB Separation",
    "Colorspace Vector"
};

// --> Form Handles
HWND hToolbarForm;
HWND hImgPathEdit;
HWND hOutPathEdit;
HWND hMskPathEdit;
HWND hImgPathButton;
HWND hMskPathButton;
HWND hOutPathButton;
HWND hAlgSelectCombo;
HWND hSepSelectCombo;
HWND hParamWindow[NUM_ALGO_OPTIONS];
HWND hMaskEditCheck;
HWND hSaveImgCheck;
HWND hHelpShowCheck;
HWND hRunButton;
HWND hResetButton;
HWND hHelpWindow;
HWND hMaskEditWindow;

// --> Parameter interface handles
HWND hGaussianOctavesLabel;
HWND hGaussianOctavesTrackbar;
HWND hFDIIKernelLabel;
HWND hFDIIKernelCombo;
HWND hFDIIIterLabel;
HWND hFDIIIterTrackbar;
HWND hBertalmioInpaintStepsLabel;
HWND hBertalmioInpaintStepsTrackbar;
HWND hBertalmioDiffuseStepsLabel;
HWND hBertalmioDiffuseStepsTrackbar;
HWND hBertalmioClimbRateLabel;
HWND hBertalmioClimbRateEdit;
HWND hBertalmioClimbRateDecayLabel;
HWND hBertalmioClimbRateDecayEdit;
HWND hTeleaRadiusLabel;
HWND hTeleaRadiusTrackbar;
HWND hNavierRadiusLabel;
HWND hNavierRadiusTrackbar;
HWND hExmpIterLabel;
HWND hExmpIterTrackbar;
HWND hExmpPatchSizeLabel;
HWND hExmpPatchSizeTrackbar;
HWND hExmpBoundaryLabel;
HWND hExmpBoundaryTrackbar;
HWND hExmpGPUCheck;

// --> Other variables
string windowLastInputFolder = "";
string windowImagePath = "";
string windowMaskPath = "";
string windowOutputPath = "";

int windowSelectedAlgorithm = 2;
int windowSelectedSeparation;
int windowSaveInterim = 0;

// --> Window interface functions

void windowError(char *msg, int value) {
    char buffer[512]; sprintf(buffer, "%s: %d", msg, value);
    MessageBox(NULL, buffer, "Error", 0);
}

void windowParametersSwitchAlgorithm() {
    for(int i = 0; i < NUM_ALGO_OPTIONS; i++) {
        ShowWindow( hParamWindow[i], ( i == windowSelectedAlgorithm ) ? SW_SHOW:SW_HIDE );
    }
    if( windowSelectedAlgorithm == 6 ) {
        ShowWindow( hParamWindow[6], SW_HIDE );
        ShowWindow( hParamWindow[5], SW_SHOW );
    }
}

double windowGetEditDouble( HWND h, double defaultVal ) {
    char buffer[1024]; double ret = 0.0;
    UIGetEditText(buffer, h, 1024);
    if( sscanf(buffer, "%lf", &ret) == 1 ) return ret;
    UISetEditTextPrintf( h, "%lf", defaultVal );
    return defaultVal;
}

void windowSetupParameters() {
    HWND hw; int cx;
    // Build parameter interface: Gaussian method
    cx = 16; hw = hParamWindow[0];
    hGaussianOctavesLabel = UILabel( "Octaves", 8, cx, 82, 16, 0, hw );
    hGaussianOctavesTrackbar = UITrackBar( 88, cx, 96, 45, ID_GAUSSIAN_OCTAVES_TRACKBAR, 
        hw, 1, MAX_OCTAVES, globalInpainter.pGaussianOctaves, 2 );
    UISetTextPrintf( hGaussianOctavesLabel, "Octaves: %d", globalInpainter.pGaussianOctaves);
    // Build parameter interface: FDII method
    cx = 16; hw = hParamWindow[1];
    hFDIIKernelLabel = UILabel( "Kernel:", 8, cx, 172, 16, 0, hw );
    hFDIIKernelCombo = UIComboBox( "", 8, cx + 16, 176, 21, ID_FDII_KERNEL_COMBO, hw );
    UIAddCombo( "Kernel 1" , hFDIIKernelCombo, 0 );
    UIAddCombo( "Kernel 2" , hFDIIKernelCombo, 1 );
    UISelCombo( hFDIIKernelCombo, globalInpainter.pFDIIKernel );
    cx += 64;
    hFDIIIterLabel = UILabel( "Iterations", 8, cx, 172, 16, 0, hw );
    hFDIIIterTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_FDII_ITER_TRACKBAR, 
        hw, 1, 200, globalInpainter.pFDIIIterations, 5 );
    UISetTextPrintf( hFDIIIterLabel, "Iterations: %d", globalInpainter.pFDIIIterations);
    // Build parameter interface: Bertalmio method
    cx = 16; hw = hParamWindow[2];
    hBertalmioInpaintStepsLabel = UILabel( "Inpaint Steps", 8, cx, 172, 16, 0, hw );
    hBertalmioInpaintStepsTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_BERTALMIO_INPSTEPS_TRACKBAR, 
        hw, 0, 50, globalInpainter.pBertalmioInpaintSteps, 1 );
    UISetTextPrintf( hBertalmioInpaintStepsLabel, "Inpaint Steps: %d", globalInpainter.pBertalmioInpaintSteps);
    cx += 64;
    hBertalmioDiffuseStepsLabel = UILabel( "Diffusion Steps", 8, cx, 172, 16, 0, hw );
    hBertalmioDiffuseStepsTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_BERTALMIO_DIFSTEPS_TRACKBAR, 
        hw, 0, 50, globalInpainter.pBertalmioDiffuseSteps, 1 );
    UISetTextPrintf( hBertalmioDiffuseStepsLabel, "Diffusion Steps: %d", globalInpainter.pBertalmioDiffuseSteps);
    cx += 64;
    hBertalmioClimbRateLabel = UILabel( "Climb Rate:", 8, cx, 92, 16, 0, hw );
    hBertalmioClimbRateEdit = UIEdit("", 104, cx, 80, 21, ID_BERTALMIO_CR_EDIT, hw);
    UISetEditTextPrintf( hBertalmioClimbRateEdit, "%lf", globalInpainter.pBertalmioClimbRate );
    cx += 24;
    hBertalmioClimbRateDecayLabel = UILabel( "Climb Decay:", 8, cx, 92, 16, 0, hw );
    hBertalmioClimbRateDecayEdit = UIEdit("", 104, cx, 80, 21, ID_BERTALMIO_CRD_EDIT, hw);
    UISetEditTextPrintf( hBertalmioClimbRateDecayEdit, "%lf", globalInpainter.pBertalmioClimbDecay );
    cx += 24;
    // Build parameter interface: Alexandru Telea - Fast Marching method
    cx = 16; hw = hParamWindow[3];
    hTeleaRadiusLabel = UILabel( "Radius", 8, cx, 172, 16, 0, hw );
    hTeleaRadiusTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_TELEA_TRACKBAR, 
        hw, 1, 10, globalInpainter.pTeleaRadius, 1 );
    UISetTextPrintf( hTeleaRadiusLabel, "Radius: %d", globalInpainter.pTeleaRadius);
    // Build parameter interface: Navier-Stokes method
    cx = 16; hw = hParamWindow[4];
    hNavierRadiusLabel = UILabel( "Radius", 8, cx, 172, 16, 0, hw );
    hNavierRadiusTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_NAVIER_TRACKBAR, 
        hw, 1, 10, globalInpainter.pNavierRadius, 1 );
    UISetTextPrintf( hNavierRadiusLabel, "Radius: %d", globalInpainter.pNavierRadius);
    // Build parameter interface: Exemplar-Based method
    cx = 16; hw = hParamWindow[5];
    hExmpIterLabel = UILabel( "Iterations", 8, cx, 172, 16, 0, hw );
    hExmpIterTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_EXMP_ITER_TRACKBAR, 
        hw, 1, 3000, globalInpainter.pExmpIterations, 100 );
    UISetTextPrintf( hExmpIterLabel, "Iterations: %d", globalInpainter.pExmpIterations);
    cx += 64;
    hExmpPatchSizeLabel = UILabel( "Patch Size", 8, cx, 172, 16, 0, hw );
    hExmpPatchSizeTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_EXMP_PTCH_TRACKBAR, 
        hw, 1, 32, globalInpainter.pExmpPatchSize, 1 );
    UISetTextPrintf( hExmpPatchSizeLabel, "Patch Size: %d", globalInpainter.pExmpPatchSize * 2 + 1);
    cx += 64;
    hExmpBoundaryLabel = UILabel( "Boundary Radius: 00", 8, cx, 172, 16, 0, hw );
    hExmpBoundaryTrackbar = UITrackBar( 8, cx + 16, 176, 45, ID_EXMP_BDRY_TRACKBAR, 
        hw, 5, 100, globalInpainter.pExmpBoundary, 1 );
    UISetTextPrintf( hExmpBoundaryLabel, "Boundary Radius: %d", globalInpainter.pExmpBoundary * 3);
    cx += 64;
    hExmpGPUCheck = UICheckBox("Use GPU Acceleration", 8, cx, 172, 17, ID_EXP_GPU_CHECK, hw, globalInpainter.pExmpGPU);
    // Build parameter interface: Local Optimisation method
    // Refresh
    windowParametersSwitchAlgorithm();
}

void windowSetup(HINSTANCE hThisInstance) {

    INITCOMMONCONTROLSEX icex;
    icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_USEREX_CLASSES;
    InitCommonControlsEx(&icex);

    MSG messages;
    WNDCLASSEX wincl;
    hinst = hThisInstance;

    // Register class
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = WINDOW_TITLE;
    wincl.lpfnWndProc = windowProcedure; 
    wincl.style = CS_DBLCLKS;
    wincl.cbSize = sizeof (WNDCLASSEX);
    wincl.hIcon = 0; wincl.hIconSm = 0;
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL; wincl.cbClsExtra = 0; wincl.cbWndExtra = 0;
    wincl.hbrBackground = (HBRUSH)GetSysColorBrush(COLOR_BTNFACE);
    if (!RegisterClassEx (&wincl)) {windowError("Register class failure!", GetLastError()); exit(1);}
    
    // Build interface: Toolbar form
    hToolbarForm = UIForm( WINDOW_TITLE, WINDOW_TITLE, CW_USEDEFAULT, CW_USEDEFAULT, 215, 646, HWND_DESKTOP, 0);
    ShowWindow(hToolbarForm, SW_SHOW);
    
    // Build interface: File loaders
    UILabel("Image:", 8, 8, 52, 16, 0, hToolbarForm );
    hImgPathEdit = UIEdit("", 8, 24, 160, 20, ID_IMGPATH_EDIT, hToolbarForm, false, false ); 
    hImgPathButton = UIButton("...", 176, 24, 24, 24, ID_IMGPATH_BUTTON, hToolbarForm);
    UILabel("Mask:", 8, 48, 52, 16, 0, hToolbarForm );
    hMskPathEdit = UIEdit("", 8, 64, 160, 20, ID_MSKPATH_EDIT, hToolbarForm, false, false ); 
    hMskPathButton = UIButton("...", 176, 64, 24, 24, ID_MSKPATH_BUTTON, hToolbarForm);
    UILabel("Output:", 8, 88, 52, 16, 0, hToolbarForm );
    hOutPathEdit = UIEdit("", 8, 104, 160, 20, ID_OUTPATH_EDIT, hToolbarForm, false, false ); 
    hOutPathButton = UIButton("...", 176, 104, 24, 24, ID_OUTPATH_BUTTON, hToolbarForm);
    
    // Build interface: Combobox selectors
    hAlgSelectCombo = UIComboBox("", 8, 128, 192, 21, ID_ALG_COMBO, hToolbarForm);
    hSepSelectCombo = UIComboBox("", 8, 152, 192, 21, ID_SEP_COMBO, hToolbarForm);

    for(int i = 0; i < NUM_ALGO_OPTIONS; i++)
        UIAddCombo( comboAlgorithmOptions[i], hAlgSelectCombo, i );
    for(int i = 0; i < NUM_SEP_OPTIONS; i++)
        UIAddCombo( comboSeparationOptions[i], hSepSelectCombo, i );

    UISelCombo( hAlgSelectCombo, windowSelectedAlgorithm );
    UISelCombo( hSepSelectCombo, windowSelectedSeparation );
    
    // Build interface: Group boxes
    for(int i = 0; i < NUM_ALGO_OPTIONS; i++) {
        hParamWindow[i] = UIWindow( "Parameters", WINDOW_TITLE, 8, 176, 192, 232, hToolbarForm);
    }
    windowParametersSwitchAlgorithm();

    UIGroupBox("Parameters", 8, 176, 192, 232, -1, hToolbarForm);
    UIGroupBox("Control", 8, 416, 192, 96, -1, hToolbarForm);

    // Build interface: Control Checks
    hMaskEditCheck = UICheckBox( "Mask edit mode", 16, 436, 163, 17, ID_MASK_CHECK, hToolbarForm, false );
    hSaveImgCheck = UICheckBox( "Dump interm. images", 16, 460, 173, 17, ID_SAVE_CHECK, hToolbarForm, false );
    hHelpShowCheck = UICheckBox( "Show help window", 16, 484, 173, 17, ID_HELP_CHECK, hToolbarForm, false );

    // Build interface: Run & Save button
    hRunButton = UIButton("Run", 8, 520, 192, 40, ID_RUN_BUTTON, hToolbarForm);
    hResetButton = UIButton("Reset", 8, 568, 192, 40, ID_RESET_BUTTON, hToolbarForm);

    hHelpWindow  = UIWindow( "Help", WINDOW_TITLE, 8, 408, 192, 96, hToolbarForm);
    
    maskBuildInterface();
    windowSetupParameters();
}

bool windowOpenFileDialog( char *dest) {
    if(!dest) return false;
    dest[0] = '\0';
    OPENFILENAME ofn = UIGetOFN(dest, 
        "BMP Bitmap Images\0*.bmp\0JPEG Images\0*.jpg\0PNG Images\0*.png\0\0", 
        "bmp", hToolbarForm, windowLastInputFolder);
    return (GetOpenFileName(&ofn));
}

bool windowSaveFileDialog( char *dest) {
    if(!dest) return false;
    dest[0] = '\0';
    OPENFILENAME ofn = UIGetOFN(dest, 
        "BMP Bitmap Images\0*.bmp\0JPEG Images\0*.jpg\0PNG Images\0*.png\0\0", 
        "bmp", hToolbarForm, windowLastInputFolder);
    return (GetSaveFileName(&ofn));
}

void windowUpdateResult() {
    globalInpainter.combine();
    cvSetMouseCallback( "Inpainter Image Output" ,&maskMouseEvent, 0 );
    if( globalInpainter[IMGI_RESULT] != NULL ) 
        iDisplay( "Inpainter Image Output", globalInpainter[IMGI_RESULT], globalInpainter[IMGI_MASK], 
            programMaskEdit, false, "", maskBrushState == BRUSH_SELECT || maskBrushState == BRUSH_FILL);
}

void windowLoadImageMask() { 
    globalInpainter.load( windowImagePath.c_str(), windowMaskPath.c_str() ); 
    windowUpdateResult();
}
void windowPrepareInpaint() {
    globalInpainter.methodSeparation = windowSelectedSeparation;
    globalInpainter.methodInpaint = windowSelectedAlgorithm;
    globalInpainter.pBertalmioClimbRate = windowGetEditDouble( hBertalmioClimbRateEdit, 0.02 );
    globalInpainter.pBertalmioClimbDecay = windowGetEditDouble( hBertalmioClimbRateDecayEdit, 0.98922 );
    globalInpainter.separate();
    globalInpainter.prepare();
    globalInpainter.combine();
    switch( windowSelectedAlgorithm ) {
        case 0: globalInpainter.methodInpaint = INP_GAUSSIANBLUR; break;
        case 1: globalInpainter.methodInpaint = INP_FDII; break;
        case 2: globalInpainter.methodInpaint = INP_BERTALMIO; break;
        case 3: globalInpainter.methodInpaint = INP_TELEA; break;
        case 4: globalInpainter.methodInpaint = INP_NAVIER; break;
        case 5: globalInpainter.methodInpaint = INP_EXEMPLAR; break;
        case 6: globalInpainter.methodInpaint = INP_LOCAL; break;
    }
    if( globalInpainter.RGBCapableMethod() ) {
        windowSelectedSeparation = SEP_RGB;
        UISelCombo( hSepSelectCombo, windowSelectedSeparation );
    }
    windowUpdateResult();
}

void windowChangeProgramState( int newState ) {
    programState = newState;
    if( newState == STATE_RUN ) {
        UISetText( hRunButton, "Stop");
    } else if( newState == STATE_FREE ) {
        UISetText( hRunButton, "Run");
    }
}

void windowHelpMessage( const char* msg ) {
    //MessageBox(NULL, msg, "Message", 0);
}


void windowUpdate() {
    if(programState == STATE_RUN) windowUpdateResult();
}