#include "Vector.h"

// =======> Vector2D Methods

Vector2D::Vector2D() {
    x = 0.0; y = 0.0;
}
Vector2D::Vector2D(double x, double y) {
    this->x = x; this->y = y;
}
Vector2D::~Vector2D() {}


Vector2D Vector2D::operator + ( const Vector2D& v ) {
    return Vector2D( this->x + v.x, this->y + v.y );
}

Vector2D Vector2D::operator - ( const Vector2D& v ) {
    return Vector2D( this->x - v.x, this->y - v.y );
}

Vector2D Vector2D::operator + ( const double s ) {
    return Vector2D( this->x + s, this->y + s );
}

Vector2D Vector2D::operator - ( const double s ) {
    return Vector2D( this->x - s, this->y - s );
}

Vector2D Vector2D::operator * ( const double s ) {
    return Vector2D( this->x * s, this->y * s );
}

Vector2D Vector2D::operator / ( const double s ) {
    if( s == 0.0 ) return (*this);
    return Vector2D( this->x / s, this->y / s );
}

double Vector2D::operator * ( const Vector2D& v ) {
    return (this->x * v.x) + (this->y * v.y);
}


void Vector2D::operator += ( const Vector2D& v ) {
    this->x += v.x; this->y += v.y;
}

void Vector2D::operator -= ( const Vector2D& v ) {
    this->x -= v.x; this->y -= v.y;
}

void Vector2D::operator += ( const double s ) {
    this->x += s; this->y += s;
}

void Vector2D::operator -= ( const double s ) {
    this->x -= s; this->y -= s;
}

void Vector2D::operator *= ( const double s ) {
    this->x *= s; this->y *= s;
}

void Vector2D::operator /= ( const double s ) {
    if(s == 0) return;
    this->x /= s; this->y /= s;
}


double Vector2D::length() {
    return sqrt((x*x)+(y*y));
}

void Vector2D::normalize() {
    double len = this->length();
    if( len <= 0.0 ) return;
    this->x /= len; this->y /= len;
}

// =======> Functions

double crossProduct( Vector2D u, Vector2D v ) {
    return ((u.x*v.y) - (u.y*v.x));
}